@@ -3,62 +3,68 @@ use warnings;
 no warnings 'uninitialized';
 
 package Form::Diva;
+$Form::Diva::VERSION = '0.05'; # TRIAL
+# ABSTRACT: Generate HTML5 form label and input fields
 
-# use 5.014;
-
-our $VERSION = '0.0103';    # VERSION
-
-=head1 NAME
- 
-Form::Diva - Generate HTML5 form label and input fields
-
-=head1 VERSION
- 
-version 0.0103
- 
-=head1 SYNOPSIS
- 
-Generate Form Label and Input Tags from a simple data structure.
-Simplify form code in your views without replacing it without a lot of even
-uglier Perl Code in your Controller. 
-
-Drastically reduce form clutter in your View Templates with small Data Structures in your Controller or Model!
-
-=head1 DESCRIPTION
-
-Create a new instance of Form::Diva from an array_ref of hash_refs describing each field of your form. The most common elements have a single letter abbreviation to reduce typing.
-
-Return a similar structure of the label and input elements ready for inclusion in a web page by your templating system.
-
-=head1 USAGE
-
-
-
-=head1 Bugs
-
-
-
-=cut
+use Storable qw(dclone);
 
 sub new {
     my $class = shift;
     my $self  = {@_};
     bless $self, $class;
-    $self->{class}   = $class;
-    $self->{FormMap} = &_expandshortcuts( $self->{form} );
+    $self->{class} = $class;
+    unless ( $self->{input_class} ) { die 'input_class is required.' }
+    unless ( $self->{label_class} ) { die 'label_class is required.' }
+    ( $self->{FormMap}, $self->{FormHash} )
+        = &_expandshortcuts( $self->{form} );
     return $self;
 }
 
+sub clone {
+    my $self = shift ;
+    my $args = shift ;
+    my $new = {};
+    my $class = 'Form::Diva';
+    $new->{FormHash}    = dclone $self->{FormHash};
+    $new->{input_class} = $args->{input_class} || $self->{input_class};
+    $new->{label_class} = $args->{label_class} || $self->{label_class};
+    $new->{form_name}   = $args->{form_name}   || $self->{form_name};
+    if( $args->{neworder} ){
+            my @reordered = map { $new->{FormHash}->{$_} } @{$args->{neworder}};
+            $new->{FormMap} = \@reordered ;
+        }
+    else { $new->{FormMap} = dclone $self->{FormMap} ; }        
+    bless $new, $class ;
+    return $new ;
+}
+
+# so far diva hasn't needed the form name
+# sub form_name {
+#     my $self = shift ;
+#     return $self->{form_name};
+# }
+
+sub input_class {
+    my $self = shift ;
+    return $self->{input_class};
+}
+
+sub label_class {
+    my $self = shift ;
+    return $self->{label_class};
+}
+
 # specification calls for single letter shortcuts on all fields
 # these all need to expand to the long form.
 sub _expandshortcuts {
     my %DivaShortMap = (
         qw /
-            n name t type i id e extra l label p placeholder
-            d default v values value values c class/
+            n name t type i id e extra x extra l label p placeholder
+            d default v values c class /
     );
     my %DivaLongMap = map { $DivaShortMap{$_}, $_ } keys(%DivaShortMap);
-    my $FormMap = shift;
+    my $FormHash    = {};
+    my $FormMap     = shift;
     foreach my $formfield ( @{$FormMap} ) {
         foreach my $tag ( keys %{$formfield} ) {
             if ( $DivaShortMap{$tag} ) {
@@ -66,10 +72,11 @@ sub _expandshortcuts {
                     = delete $formfield->{$tag};
             }
         }
-        unless ( $formfield->{type} ) { $formfield->{type} = 'text'  }
-        unless ( $formfield->{name} ) { die "fields must have names" }     
+        unless ( $formfield->{type} ) { $formfield->{type} = 'text' }
+        unless ( $formfield->{name} ) { die "fields must have names" }
+        $FormHash->{ $formfield->{name} } = $formfield;
     }
-    return $FormMap;
+    return ( $FormMap, $FormHash );
 }
 
 # given a field returns either the default field class="string"
@@ -124,8 +131,7 @@ sub _label {
     my $fname       = $field->{name};
     my $label_class = $self->{label_class};
     my $label_tag   = $field->{label} || ucfirst($fname);
-    return
-          qq|<LABEL for="$fname" class="$label_class">|
+    return qq|<LABEL for="$fname" class="$label_class">|
         . qq|$label_tag</LABEL>|;
 }
 
@@ -136,11 +142,11 @@ sub _input {
     my %B     = $self->_field_bits( $field, $data );
     my $input = '';
     if ( $B{textarea} ) {
-        $input = qq|<TEXTAREA $B{name} $B{id}"
+        $input = qq|<TEXTAREA $B{name} $B{id}
         $B{input_class} $B{placeholder} $B{extra} >$B{rawvalue}</TEXTAREA>|;
     }
     else {
-        $input .= qq|<INPUT $B{type} $B{name} $B{id}"
+        $input .= qq|<INPUT $B{type} $B{name} $B{id}
         $B{input_class} $B{placeholder} $B{extra} $B{value} >|;
     }
     $input =~ s/\s+/ /g;     # remove extra whitespace.
@@ -152,23 +158,28 @@ sub _input {
 # this needs to be implemented after data is being handled because
 # default is irrelevant if there is data.
 
-sub _radiocheck {            # field, input_class, data;
-    my $self        = shift;
-    my $field       = shift;
-    my $input_class = shift;
-    my $data        = shift;
-    my $output      = '';
-    my $extra       = $field->{extra} || "";
-    my $default     = $field->{default}
+sub _radiocheck {    # field, input_class, data;
+    my $self           = shift;
+    my $field          = shift;
+    my $data           = shift;
+    my $replace_fields = shift;
+    my $output         = '';
+    my $input_class    = $self->_class_input($field);
+    my $extra          = $field->{extra} || "";
+    my $default        = $field->{default}
         ? do {
         if   ($data) {undef}
         else         { $field->{default} }
         }
         : undef;
-    foreach my $val ( @{ $field->{values} } ) {
+    my @values
+        = $replace_fields
+        ? @{$replace_fields}
+        : @{ $field->{values} };    
+    foreach my $val ( @values ) {
         my ( $value, $v_lab ) = ( split( /\:/, $val ), $val );
         my $checked = '';
-        if    ( $data    eq $value ) { $checked = 'checked ' }
+        if    ( $data eq $value )    { $checked = 'checked ' }
         elsif ( $default eq $value ) { $checked = 'checked ' }
         $output
             .= qq!<input type="$field->{type}" $input_class $extra name="$field->{name}" value="$value" $checked>$v_lab<br>\n!;
@@ -176,9 +187,40 @@ sub _radiocheck {            # field, input_class, data;
     return $output;
 }
 
+sub _select {    # field, input_class, data;
+    my $self           = shift;
+    my $field          = shift;
+    my $data           = shift;
+    my $replace_fields = shift;
+    my $class          = $self->_class_input($field);
+    my $extra          = $field->{extra} || "";
+    my $id = $field->{id} ? qq!id="$field->{id}"! : qq!id="$field->{name}"!;
+    my @values
+        = $replace_fields
+        ? @{$replace_fields}
+        : @{ $field->{values} };
+    my $default = $field->{default}
+        ? do {
+        if   ($data) {undef}
+        else         { $field->{default} }
+        }
+        : undef;
+    my $output = qq|<SELECT name="$field->{name}" $id $extra $class>\n|;
+    foreach my $val (@values) {
+        my ( $value, $v_lab ) = ( split( /\:/, $val ), $val );
+        my $selected = '';
+        if    ( $data eq $value )    { $selected = 'selected ' }
+        elsif ( $default eq $value ) { $selected = 'selected ' }
+        $output .= qq| <option value="$value" $selected>$v_lab</option>\n|;
+    }
+    $output .= '</SELECT>';
+    return $output;
+}
+
 sub generate {
-    my $self = shift;
-    my $data = shift;
+    my $self    = shift;
+    my $data    = shift;
+    my $overide = shift;
     unless ( keys %{$data} ) { $data = undef }
     my @generated = ();
     foreach my $field ( @{ $self->{FormMap} } ) {
@@ -186,8 +228,15 @@ sub generate {
         if ( $field->{type} eq 'radio' || $field->{type} eq 'checkbox' ) {
             $input = $self->_radiocheck(
                 $field,
-                $self->_class_input($field),
-                $data->{ $field->{name} }
+                $data->{ $field->{name} },
+                $overide->{ $field->{name} },
+            );
+        }
+        elsif ( $field->{type} eq 'select' || $field->{type} eq 'datalist' ) {
+            $input = $self->_select(
+                $field,
+                $data->{ $field->{name} },
+                $overide->{ $field->{name} },
             );
         }
         else {
@@ -205,43 +254,3 @@ sub generate {
 }
 
 1;
-
-=head1 AUTHOR
-
-John Karr, C<brainbuz at brainbuz.org>
-
-=head1 BUGS
-
-Please report any bugs or feature requests through the web interface at L<https://bitbucket.org/brainbuz/formdiva/issues>.  I will be notified, and then you'll automatically be notified of progress on your bug as I make changes.
-
-=head1 SUPPORT
-
-You can find documentation for this module with the perldoc command.
-
-    perldoc Form::Diva
-
-You can also look for information at:
-
-=head1 ACKNOWLEDGEMENTS
-
-=head1 LICENSE AND COPYRIGHT
-
-Copyright 2014 John Karr.
-
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program.  If not, see L<http://www.gnu.org/licenses/>.
-
-=cut
-
-1;
-
